from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M50SVH10


class BTMinerM50SVH10(M5X, M50SVH10):
    pass


from pyasic.miners.device.models import M50SVH20


class BTMinerM50SVH20(M5X, M50SVH20):
    pass


from pyasic.miners.device.models import M50SVH30


class BTMinerM50SVH30(M5X, M50SVH30):
    pass


from pyasic.miners.device.models import M50SVH40


class BTMinerM50SVH40(M5X, M50SVH40):
    pass


from pyasic.miners.device.models import M50SVH50


class BTMinerM50SVH50(M5X, M50SVH50):
    pass


from pyasic.miners.device.models import M50SVJ10


class BTMinerM50SVJ10(M5X, M50SVJ10):
    pass


from pyasic.miners.device.models import M50SVJ20


class BTMinerM50SVJ20(M5X, M50SVJ20):
    pass


from pyasic.miners.device.models import M50SVJ30


class BTMinerM50SVJ30(M5X, M50SVJ30):
    pass


from pyasic.miners.device.models import M50SVJ40


class BTMinerM50SVJ40(M5X, M50SVJ40):
    pass


from pyasic.miners.device.models import M50SVJ50


class BTMinerM50SVJ50(M5X, M50SVJ50):
    pass


from pyasic.miners.device.models import M50SVK10


class BTMinerM50SVK10(M5X, M50SVK10):
    pass


from pyasic.miners.device.models import M50SVK20


class BTMinerM50SVK20(M5X, M50SVK20):
    pass


from pyasic.miners.device.models import M50SVK30


class BTMinerM50SVK30(M5X, M50SVK30):
    pass


from pyasic.miners.device.models import M50SVK50


class BTMinerM50SVK50(M5X, M50SVK50):
    pass


from pyasic.miners.device.models import M50SVK60


class BTMinerM50SVK60(M5X, M50SVK60):
    pass


from pyasic.miners.device.models import M50SVK70


class BTMinerM50SVK70(M5X, M50SVK70):
    pass


from pyasic.miners.device.models import M50SVK80


class BTMinerM50SVK80(M5X, M50SVK80):
    pass


from pyasic.miners.device.models import M50SVL20


class BTMinerM50SVL20(M5X, M50SVL20):
    pass


from pyasic.miners.device.models import M50SVL30


class BTMinerM50SVL30(M5X, M50SVL30):
    pass
