from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M30SPlusPlusV10


class BTMinerM30SPlusPlusV10(M3X, M30SPlusPlusV10):
    pass


from pyasic.miners.device.models import M30SPlusPlusV20


class BTMinerM30SPlusPlusV20(M3X, M30SPlusPlusV20):
    pass


from pyasic.miners.device.models import M30SPlusPlusVE30


class BTMinerM30SPlusPlusVE30(M3X, M30SPlusPlusVE30):
    pass


from pyasic.miners.device.models import M30SPlusPlusVE40


class BTMinerM30SPlusPlusVE40(M3X, M30SPlusPlusVE40):
    pass


from pyasic.miners.device.models import M30SPlusPlusVE50


class BTMinerM30SPlusPlusVE50(M3X, M30SPlusPlusVE50):
    pass


from pyasic.miners.device.models import M30SPlusPlusVF40


class BTMinerM30SPlusPlusVF40(M3X, M30SPlusPlusVF40):
    pass


from pyasic.miners.device.models import M30SPlusPlusVG30


class BTMinerM30SPlusPlusVG30(M3X, M30SPlusPlusVG30):
    pass


from pyasic.miners.device.models import M30SPlusPlusVG40


class BTMinerM30SPlusPlusVG40(M3X, M30SPlusPlusVG40):
    pass


from pyasic.miners.device.models import M30SPlusPlusVG50


class BTMinerM30SPlusPlusVG50(M3X, M30SPlusPlusVG50):
    pass


from pyasic.miners.device.models import M30SPlusPlusVH10


class BTMinerM30SPlusPlusVH10(M3X, M30SPlusPlusVH10):
    pass


from pyasic.miners.device.models import M30SPlusPlusVH100


class BTMinerM30SPlusPlusVH100(M3X, M30SPlusPlusVH100):
    pass


from pyasic.miners.device.models import M30SPlusPlusVH110


class BTMinerM30SPlusPlusVH110(M3X, M30SPlusPlusVH110):
    pass


from pyasic.miners.device.models import M30SPlusPlusVH20


class BTMinerM30SPlusPlusVH20(M3X, M30SPlusPlusVH20):
    pass


from pyasic.miners.device.models import M30SPlusPlusVH30


class BTMinerM30SPlusPlusVH30(M3X, M30SPlusPlusVH30):
    pass


from pyasic.miners.device.models import M30SPlusPlusVH40


class BTMinerM30SPlusPlusVH40(M3X, M30SPlusPlusVH40):
    pass


from pyasic.miners.device.models import M30SPlusPlusVH50


class BTMinerM30SPlusPlusVH50(M3X, M30SPlusPlusVH50):
    pass


from pyasic.miners.device.models import M30SPlusPlusVH60


class BTMinerM30SPlusPlusVH60(M3X, M30SPlusPlusVH60):
    pass


from pyasic.miners.device.models import M30SPlusPlusVH70


class BTMinerM30SPlusPlusVH70(M3X, M30SPlusPlusVH70):
    pass


from pyasic.miners.device.models import M30SPlusPlusVH80


class BTMinerM30SPlusPlusVH80(M3X, M30SPlusPlusVH80):
    pass


from pyasic.miners.device.models import M30SPlusPlusVH90


class BTMinerM30SPlusPlusVH90(M3X, M30SPlusPlusVH90):
    pass


from pyasic.miners.device.models import M30SPlusPlusVI30


class BTMinerM30SPlusPlusVI30(M3X, M30SPlusPlusVI30):
    pass


from pyasic.miners.device.models import M30SPlusPlusVJ20


class BTMinerM30SPlusPlusVJ20(M3X, M30SPlusPlusVJ20):
    pass


from pyasic.miners.device.models import M30SPlusPlusVJ30


class BTMinerM30SPlusPlusVJ30(M3X, M30SPlusPlusVJ30):
    pass


from pyasic.miners.device.models import M30SPlusPlusVJ50


class BTMinerM30SPlusPlusVJ50(M3X, M30SPlusPlusVJ50):
    pass


from pyasic.miners.device.models import M30SPlusPlusVJ60


class BTMinerM30SPlusPlusVJ60(M3X, M30SPlusPlusVJ60):
    pass


from pyasic.miners.device.models import M30SPlusPlusVJ70


class BTMinerM30SPlusPlusVJ70(M3X, M30SPlusPlusVJ70):
    pass


from pyasic.miners.device.models import M30SPlusPlusVK30


class BTMinerM30SPlusPlusVK30(M3X, M30SPlusPlusVK30):
    pass


from pyasic.miners.device.models import M30SPlusPlusVK40


class BTMinerM30SPlusPlusVK40(M3X, M30SPlusPlusVK40):
    pass
