from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M54SPlusPlusVK30


class BTMinerM54SPlusPlusVK30(M5X, M54SPlusPlusVK30):
    pass


from pyasic.miners.device.models import M54SPlusPlusVL30


class BTMinerM54SPlusPlusVL30(M5X, M54SPlusPlusVL30):
    pass


from pyasic.miners.device.models import M54SPlusPlusVL40


class BTMinerM54SPlusPlusVL40(M5X, M54SPlusPlusVL40):
    pass
