from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M53SPlusVJ30


class BTMinerM53SPlusVJ30(M5X, M53SPlusVJ30):
    pass


from pyasic.miners.device.models import M53SPlusVJ40


class BTMinerM53SPlusVJ40(M5X, M53SPlusVJ40):
    pass


from pyasic.miners.device.models import M53SPlusVJ50


class BTMinerM53SPlusVJ50(M5X, M53SPlusVJ50):
    pass


from pyasic.miners.device.models import M53SPlusVK30


class BTMinerM53SPlusVK30(M5X, M53SPlusVK30):
    pass
