from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M39V10


class BTMinerM39V10(M3X, M39V10):
    pass


from pyasic.miners.device.models import M39V20


class BTMinerM39V20(M3X, M39V20):
    pass


from pyasic.miners.device.models import M39V30


class BTMinerM39V30(M3X, M39V30):
    pass
