from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M60SPlusPlusVL30


class BTMinerM60SPlusPlusVL30(M6X, M60SPlusPlusVL30):
    pass


from pyasic.miners.device.models import M60SPlusPlusVL40


class BTMinerM60SPlusPlusVL40(M6X, M60SPlusPlusVL40):
    pass
