from pyasic.miners.backends import M3X
from pyasic.miners.device.models import M31HV10


class BTMinerM31HV10(M3X, M31HV10):
    pass


from pyasic.miners.device.models import M31HV40


class BTMinerM31HV40(M3X, M31HV40):
    pass
