from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M64VL30


class BTMinerM64VL30(M6X, M64VL30):
    pass


from pyasic.miners.device.models import M64VL40


class BTMinerM64VL40(M6X, M64VL40):
    pass
