from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M66SVK20


class BTMinerM66SVK20(M6X, M66SVK20):
    pass


from pyasic.miners.device.models import M66SVK30


class BTMinerM66SVK30(M6X, M66SVK30):
    pass


from pyasic.miners.device.models import M66SVK40


class BTMinerM66SVK40(M6X, M66SVK40):
    pass


from pyasic.miners.device.models import M66SVK50


class BTMinerM66SVK50(M6X, M66SVK50):
    pass


from pyasic.miners.device.models import M66SVK60


class BTMinerM66SVK60(M6X, M66SVK60):
    pass


from pyasic.miners.device.models import M66SVL10


class BTMinerM66SVL10(M6X, M66SVL10):
    pass


from pyasic.miners.device.models import M66SVL20


class BTMinerM66SVL20(M6X, M66SVL20):
    pass


from pyasic.miners.device.models import M66SVL30


class BTMinerM66SVL30(M6X, M66SVL30):
    pass


from pyasic.miners.device.models import M66SVL40


class BTMinerM66SVL40(M6X, M66SVL40):
    pass


from pyasic.miners.device.models import M66SVL50


class BTMinerM66SVL50(M6X, M66SVL50):
    pass
