from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M60VK10


class BTMinerM60VK10(M6X, M60VK10):
    pass


from pyasic.miners.device.models import M60VK20


class BTMinerM60VK20(M6X, M60VK20):
    pass


from pyasic.miners.device.models import M60VK30


class BTMinerM60VK30(M6X, M60VK30):
    pass


from pyasic.miners.device.models import M60VK40


class BTMinerM60VK40(M6X, M60VK40):
    pass


from pyasic.miners.device.models import M60VK6A


class BTMinerM60VK6A(M6X, M60VK6A):
    pass


from pyasic.miners.device.models import M60VL10


class BTMinerM60VL10(M6X, M60VL10):
    pass


from pyasic.miners.device.models import M60VL20


class BTMinerM60VL20(M6X, M60VL20):
    pass


from pyasic.miners.device.models import M60VL30


class BTMinerM60VL30(M6X, M60VL30):
    pass


from pyasic.miners.device.models import M60VL40


class BTMinerM60VL40(M6X, M60VL40):
    pass


from pyasic.miners.device.models import M60VL50


class BTMinerM60VL50(M6X, M60VL50):
    pass
