from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M66VK20


class BTMinerM66VK20(M6X, M66VK20):
    pass


from pyasic.miners.device.models import M66VK30


class BTMinerM66VK30(M6X, M66VK30):
    pass


from pyasic.miners.device.models import M66VL20


class BTMinerM66VL20(M6X, M66VL20):
    pass


from pyasic.miners.device.models import M66VL30


class BTMinerM66VL30(M6X, M66VL30):
    pass
