from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M56SVH30


class BTMinerM56SVH30(M5X, M56SVH30):
    pass


from pyasic.miners.device.models import M56SVJ30


class BTMinerM56SVJ30(M5X, M56SVJ30):
    pass


from pyasic.miners.device.models import M56SVJ40


class BTMinerM56SVJ40(M5X, M56SVJ40):
    pass
