from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M63SPlusVK30


class BTMinerM63SPlusVK30(M6X, M63SPlusVK30):
    pass


from pyasic.miners.device.models import M63SPlusVL10


class BTMinerM63SPlusVL10(M6X, M63SPlusVL10):
    pass


from pyasic.miners.device.models import M63SPlusVL20


class BTMinerM63SPlusVL20(M6X, M63SPlusVL20):
    pass


from pyasic.miners.device.models import M63SPlusVL30


class BTMinerM63SPlusVL30(M6X, M63SPlusVL30):
    pass


from pyasic.miners.device.models import M63SPlusVL50


class BTMinerM63SPlusVL50(M6X, M63SPlusVL50):
    pass
