from pyasic.miners.backends import M5X
from pyasic.miners.device.models import M50SPlusVH30


class BTMinerM50SPlusVH30(M5X, M50SPlusVH30):
    pass


from pyasic.miners.device.models import M50SPlusVH40


class BTMinerM50SPlusVH40(M5X, M50SPlusVH40):
    pass


from pyasic.miners.device.models import M50SPlusVJ30


class BTMinerM50SPlusVJ30(M5X, M50SPlusVJ30):
    pass


from pyasic.miners.device.models import M50SPlusVJ40


class BTMinerM50SPlusVJ40(M5X, M50SPlusVJ40):
    pass


from pyasic.miners.device.models import M50SPlusVJ60


class BTMinerM50SPlusVJ60(M5X, M50SPlusVJ60):
    pass


from pyasic.miners.device.models import M50SPlusVK10


class BTMinerM50SPlusVK10(M5X, M50SPlusVK10):
    pass


from pyasic.miners.device.models import M50SPlusVK20


class BTMinerM50SPlusVK20(M5X, M50SPlusVK20):
    pass


from pyasic.miners.device.models import M50SPlusVK30


class BTMinerM50SPlusVK30(M5X, M50SPlusVK30):
    pass


from pyasic.miners.device.models import M50SPlusVL10


class BTMinerM50SPlusVL10(M5X, M50SPlusVL10):
    pass


from pyasic.miners.device.models import M50SPlusVL20


class BTMinerM50SPlusVL20(M5X, M50SPlusVL20):
    pass


from pyasic.miners.device.models import M50SPlusVL30


class BTMinerM50SPlusVL30(M5X, M50SPlusVL30):
    pass
