from .M50 import (
    BTMinerM50VE30,
    BTMinerM50VG30,
    BTMinerM50VH10,
    BTMinerM50VH20,
    BTMinerM50VH30,
    BTMinerM50VH40,
    BTMinerM50VH50,
    BTMinerM50VH60,
    BTMinerM50VH70,
    BTMinerM50VH80,
    BTMinerM50VH90,
    BTMinerM50VJ10,
    BTMinerM50VJ20,
    BTMinerM50VJ30,
    BTMinerM50VJ40,
    BTMinerM50VJ60,
    BTMinerM50VK40,
    BTMinerM50VK50,
)
from .M50S import (
    BTMinerM50SVH10,
    BTMinerM50SVH20,
    BTMinerM50SVH30,
    BTMinerM50SVH40,
    BTMinerM50SVH50,
    BTMinerM50SVJ10,
    BTMinerM50SVJ20,
    BTMinerM50SVJ30,
    BTMinerM50SVJ40,
    BTMinerM50SVJ50,
    BTMinerM50SVK10,
    BTMinerM50SVK20,
    BTMinerM50SVK30,
    BTMinerM50SVK50,
    BTMinerM50SVK60,
    BTMinerM50SVK70,
    BTMinerM50SVK80,
    BTMinerM50SVL20,
    BTMinerM50SVL30,
)
from .M50S_Plus import (
    BTMinerM50SPlusVH30,
    BTMinerM50SPlusVH40,
    BTMinerM50SPlusVJ30,
    BTMinerM50SPlusVJ40,
    BTMinerM50SPlusVJ60,
    BTMinerM50SPlusVK10,
    BTMinerM50SPlusVK20,
    BTMinerM50SPlusVK30,
    BTMinerM50SPlusVL10,
    BTMinerM50SPlusVL20,
    BTMinerM50SPlusVL30,
)
from .M50S_Plus_Plus import (
    BTMinerM50SPlusPlusVK10,
    BTMinerM50SPlusPlusVK20,
    BTMinerM50SPlusPlusVK30,
    BTMinerM50SPlusPlusVK40,
    BTMinerM50SPlusPlusVK50,
    BTMinerM50SPlusPlusVK60,
    BTMinerM50SPlusPlusVL20,
    BTMinerM50SPlusPlusVL30,
    BTMinerM50SPlusPlusVL40,
    BTMinerM50SPlusPlusVL50,
    BTMinerM50SPlusPlusVL60,
)
from .M52S import BTMinerM52SVK30
from .M52S_Plus_Plus import BTMinerM52SPlusPlusVL10
from .M53 import (
    BTMinerM53VH30,
    BTMinerM53VH40,
    BTMinerM53VH50,
    BTMinerM53VK30,
    BTMinerM53VK60,
)
from .M53H import BTMinerM53HVH10
from .M53S import (
    BTMinerM53SVH20,
    BTMinerM53SVH30,
    BTMinerM53SVJ30,
    BTMinerM53SVJ40,
    BTMinerM53SVK30,
)
from .M53S_Plus import (
    BTMinerM53SPlusVJ30,
    BTMinerM53SPlusVJ40,
    BTMinerM53SPlusVJ50,
    BTMinerM53SPlusVK30,
)
from .M53S_Plus_Plus import (
    BTMinerM53SPlusPlusVK10,
    BTMinerM53SPlusPlusVK20,
    BTMinerM53SPlusPlusVK30,
    BTMinerM53SPlusPlusVK50,
    BTMinerM53SPlusPlusVL10,
    BTMinerM53SPlusPlusVL30,
)
from .M54S_Plus_Plus import (
    BTMinerM54SPlusPlusVK30,
    BTMinerM54SPlusPlusVL30,
    BTMinerM54SPlusPlusVL40,
)
from .M56 import BTMinerM56VH30
from .M56S import BTMinerM56SVH30, BTMinerM56SVJ30, BTMinerM56SVJ40
from .M56S_Plus import (
    BTMinerM56SPlusVJ30,
    BTMinerM56SPlusVK30,
    BTMinerM56SPlusVK40,
    BTMinerM56SPlusVK50,
)
from .M56S_Plus_Plus import (
    BTMinerM56SPlusPlusVK10,
    BTMinerM56SPlusPlusVK30,
    BTMinerM56SPlusPlusVK40,
    BTMinerM56SPlusPlusVK50,
)
from .M59 import BTMinerM59VH30
