from pyasic.miners.backends import M6X
from pyasic.miners.device.models import M60SVK10


class BTMinerM60SVK10(M6X, M60SVK10):
    pass


from pyasic.miners.device.models import M60SVK20


class BTMinerM60SVK20(M6X, M60SVK20):
    pass


from pyasic.miners.device.models import M60SVK30


class BTMinerM60SVK30(M6X, M60SVK30):
    pass


from pyasic.miners.device.models import M60SVK40


class BTMinerM60SVK40(M6X, M60SVK40):
    pass


from pyasic.miners.device.models import M60SVL10


class BTMinerM60SVL10(M6X, M60SVL10):
    pass


from pyasic.miners.device.models import M60SVL20


class BTMinerM60SVL20(M6X, M60SVL20):
    pass


from pyasic.miners.device.models import M60SVL30


class BTMinerM60SVL30(M6X, M60SVL30):
    pass


from pyasic.miners.device.models import M60SVL40


class BTMinerM60SVL40(M6X, M60SVL40):
    pass


from pyasic.miners.device.models import M60SVL50


class BTMinerM60SVL50(M6X, M60SVL50):
    pass


from pyasic.miners.device.models import M60SVL60


class BTMinerM60SVL60(M6X, M60SVL60):
    pass


from pyasic.miners.device.models import M60SVL70


class BTMinerM60SVL70(M6X, M60SVL70):
    pass
